/*******************************************
 * Written by: Hui Fang (hfang@ece.udel.edu) 
 * Last updated: 08/16/2009
 * For Lucene-2.4.1 
 * Build Index for TREC format documents and compute average document length 
 * Index directory: "index" 
 ******************************************/ 

import org.apache.lucene.analysis.standard.StandardAnalyzer;
import org.apache.lucene.index.IndexWriter;
import org.apache.lucene.document.Document;
import org.apache.lucene.document.*;  
import org.apache.lucene.store.FSDirectory;
import org.apache.lucene.store.Directory;
import org.apache.lucene.index.*; 
import org.apache.lucene.search.Similarity;


import java.util.*;
import java.io.*;

class BuildIndex {
  public static void main(String[] args) throws IOException {
    String usage = "java " + BuildIndex.class + " <data_file>";
    if (args.length == 0) {
      System.err.println("Usage: " + usage);
      System.exit(1);
    }

    Date start = new Date();
    try {
	IndexWriter writer = new IndexWriter("index", new StandardAnalyzer(), true,IndexWriter.MaxFieldLength.UNLIMITED);
      File fin=new File(args[0]);
      FileInputStream inf = new FileInputStream(fin);
      InputStreamReader isr = new InputStreamReader(inf, "US-ASCII");
      BufferedReader in = new BufferedReader(isr);

      Document doc = new Document(); 
      String docid="";
      try {

	  while (true) {
	      String line = in.readLine();
	      if (line == null) break;
	      if (line.startsWith("<DOC>")) {
		  line = in.readLine();
		  String tempStr = line.substring(8,line.length());
		  int pos = tempStr.indexOf(' ');
		  docid = tempStr.substring(0,pos);
		  doc = new Document();
		  doc.add(new Field("DocID",docid,Field.Store.YES, Field.Index.NOT_ANALYZED));
	      }
	      else if (line.startsWith("</DOC>")) {
		  writer.addDocument(doc);
	      }
	      else {
		  doc.add(new Field("contents", line, Field.Store.YES, Field.Index.ANALYZED));
	      }
	  }


      } catch (IOException e) {
		System.out.println( "caught a " + e.getClass() +
                         "\n with message: " + e.getMessage());
      }

      int numIndexed = writer.maxDoc(); 

      writer.optimize();
      writer.close();

      Date end = new Date();

      System.out.print(end.getTime() - start.getTime());
      System.out.println(" total milliseconds");
      System.out.println(numIndexed + " documents are indexed");

    } catch (IOException e) {
      System.out.println(" caught a " + e.getClass() +
       "\n with message: " + e.getMessage());
    }


    Directory dir = null;
    try {
        dir = FSDirectory.getDirectory("index");
    } catch (IOException e) {
        System.out.println(" Can not open directory index!");
        System.exit(1);
    }
    SegmentInfos segInfo = new SegmentInfos();
    segInfo.read(dir);
    int numSegments = segInfo.size();
    BufferedWriter out = new BufferedWriter (new FileWriter("index/LengthInfo.txt"));
    for (int i=0; i<numSegments; i++) {
        SegmentInfo info=segInfo.info(i);
        SegmentReader reader=null;
        reader = SegmentReader.get(info);
        Collection fieldNames = reader.getFieldNames(IndexReader.FieldOption.ALL);
        Iterator it = fieldNames.iterator();
        byte [] b = new byte[reader.maxDoc()];
        while (it.hasNext()) {
            String fieldName = (String) it.next();
            reader.norms(fieldName, b, 0);
            float sum=0.0f;
            for (int j=0; j<b.length; j++){
                float dl = 1.0f / Similarity.decodeNorm(b[j]) / Similarity.decodeNorm(b[j]);
                sum += dl;
            }
            out.write(fieldName+" "+sum+" "+ b.length + "\n");
        }
        reader.close();
    }

    out.close();



  }
}
